From 7d483622778099f4f7119a3d4f462684c210db32 Mon Sep 17 00:00:00 2001
From: Arina Neshlyaeva <arina.neshlyaeva@intel.com>
Date: Tue, 6 Sep 2022 15:35:24 -0700
Subject: [PATCH] Added AddressInfo class

---
 src/ctx.cpp | 22 ++++++++++++++++++++++
 src/ctx.h   | 27 ++++++++++++++++++++++++++-
 2 files changed, 48 insertions(+), 1 deletion(-)

diff --git a/src/ctx.cpp b/src/ctx.cpp
index 03d58b9e3..1b9c6c3bc 100644
--- a/src/ctx.cpp
+++ b/src/ctx.cpp
@@ -218,6 +218,28 @@ CFInfo *CFInfo::GetSwitch(bool isUniform, bool isUniformEmulated, llvm::BasicBlo
 
 ///////////////////////////////////////////////////////////////////////////
 
+AddressInfo::AddressInfo(llvm::Value *p, llvm::Type *t) : pointer(p), elementType(t), ispcType(nullptr) {
+    Assert(pointer != nullptr && "Pointer cannot be null");
+    Assert(elementType != nullptr && "Element type cannot be null");
+}
+AddressInfo::AddressInfo(llvm::Value *p, const Type *t) : pointer(p), ispcType(t) {
+    Assert(pointer != nullptr && "Pointer cannot be null");
+    Assert(ispcType != nullptr && "ISPC type cannot be null");
+    // Get LLVM pointer element type based on ISPC type.
+    // TODO: need more testing
+    if (CastType<ReferenceType>(t) != nullptr) {
+        PointerType *pType = PointerType::GetUniform(t->GetReferenceTarget());
+        elementType = pType->GetBaseType()->LLVMStorageType(g->ctx);
+    } else if (CastType<PointerType>(t) != nullptr) {
+        elementType = t->GetBaseType()->LLVMStorageType(g->ctx);
+    } else {
+        elementType = t->LLVMStorageType(g->ctx);
+    }
+    Assert(elementType != nullptr && "Element type cannot be null");
+}
+
+///////////////////////////////////////////////////////////////////////////
+
 FunctionEmitContext::FunctionEmitContext(Function *func, Symbol *funSym, llvm::Function *lf, SourcePos firstStmtPos) {
     function = func;
     llvmFunction = lf;
diff --git a/src/ctx.h b/src/ctx.h
index 08daa5761..a842d96d7 100644
--- a/src/ctx.h
+++ b/src/ctx.h
@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2010-2021, Intel Corporation
+  Copyright (c) 2010-2022, Intel Corporation
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,31 @@ namespace ispc {
 
 struct CFInfo;
 
+///////////////////////////////////////////////////////////////////////////
+/** AddressInfo is a helper class to work with pointers.
+    It keeps llvm pointer, llvm element type, and ISPC type.
+*/
+class AddressInfo {
+  public:
+    AddressInfo(llvm::Value *p, llvm::Type *t);
+    AddressInfo(llvm::Value *p, const Type *t);
+    llvm::Value *getPointer() const { return pointer; }
+
+    // Return the type of the pointer value.
+    llvm::PointerType *getType() const { return llvm::cast<llvm::PointerType>(getPointer()->getType()); }
+
+    // Return the type of the values stored in this address.
+    llvm::Type *getElementType() const { return elementType; }
+
+    // Return the address space that this address resides in.
+    unsigned getAddressSpace() const { return getType()->getAddressSpace(); }
+
+  private:
+    llvm::Value *pointer;
+    llvm::Type *elementType;
+    const Type *ispcType;
+};
+
 /** FunctionEmitContext is one of the key classes in ispc; it is used to
     help with emitting the intermediate representation of a function during
     compilation.  It carries information the current program context during
